/* *****************************************************************************
 *
 * Filename: JustificationEditor.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  02 Februrary 2003
 *
 * Purpose:  Simple bean for selecting an anchor for GridBadConstraints.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.awt.event.*;
import java.awt.GridBagConstraints;
import java.util.ResourceBundle;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.KeyStroke;
import javax.swing.JToggleButton;
import javax.swing.event.EventListenerList;

/**
 *
 * @author  Robert
 */
public class JustificationEditor
    extends EditorBase
{
    // Statics
    private static ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    
    // Properties
    private int _value = -1;
    
    
    /** Creates new form BeanForm */
    public JustificationEditor()
    {
        initComponents();
        
        // Add actions to set 'value' with appropriate anchor constant
        leftBtn.addActionListener( new JustifyAction( Title.JUSTIFY_LEFT ) );
        centerBtn.addActionListener( new JustifyAction( Title.JUSTIFY_CENTER ) );
        rightBtn.addActionListener( new JustifyAction( Title.JUSTIFY_RIGHT ) );
        
        // Create actions for moving focus round the grid
        ActionMap actions = this.getActionMap();
        actions.put( "navigateLeft",  new GridNavigateAction( this, -1, 0 ) );
        actions.put( "navigateRight", new GridNavigateAction( this, +1, 0 ) );
        
        // Bind actions to cursor-keys 
        InputMap input = this.getInputMap( WHEN_ANCESTOR_OF_FOCUSED_COMPONENT );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_LEFT,0), "navigateLeft" );
        input.put( KeyStroke.getKeyStroke(KeyEvent.VK_RIGHT,0), "navigateRight" );
        
        // Add a FocusListener to move the initial focus onto the selected button
        this.addFocusListener( new FocusAdapter() {
            public void focusGained( FocusEvent e ) {
                JToggleButton button = buttonFromValue( _value );
                if( button != null )
                    button.requestFocus();
            }
        } );
        
        // Set up accessible names for all the buttons
        leftBtn.getAccessibleContext().setAccessibleName( resources.getString( "Justify_Left_Name" ) );
        centerBtn.getAccessibleContext().setAccessibleName( resources.getString( "Justify_Center_Name" ) );
        rightBtn.getAccessibleContext().setAccessibleName( resources.getString( "Justify_Right_Name" ) );
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()//GEN-BEGIN:initComponents
    {
        justifyBtnGrp = new javax.swing.ButtonGroup();
        leftBtn = new javax.swing.JToggleButton();
        centerBtn = new javax.swing.JToggleButton();
        rightBtn = new javax.swing.JToggleButton();

        setLayout(new java.awt.GridLayout(1, 3, 1, 1));

        leftBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/west.gif")));
        leftBtn.setToolTipText(resources.getString("Justify_Left_Tooltip"));
        justifyBtnGrp.add(leftBtn);
        leftBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(leftBtn);

        centerBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/center.gif")));
        centerBtn.setToolTipText(resources.getString("Justify_Center_Tooltip"));
        justifyBtnGrp.add(centerBtn);
        centerBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(centerBtn);

        rightBtn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/com/tsac/calmaker/beans/icons/east.gif")));
        rightBtn.setToolTipText(resources.getString("Justify_Right_Tooltip"));
        justifyBtnGrp.add(rightBtn);
        rightBtn.setMargin(new java.awt.Insets(1, 1, 1, 1));
        add(rightBtn);

    }//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JToggleButton rightBtn;
    private javax.swing.ButtonGroup justifyBtnGrp;
    private javax.swing.JToggleButton leftBtn;
    private javax.swing.JToggleButton centerBtn;
    // End of variables declaration//GEN-END:variables

    
    public void setValue( int value )
    {
        int oldval = _value;
        
        // De-bounce - otherwise we'll loop indefinitely
        if( value != oldval )
        {
            _value = value;
            
            JToggleButton button = buttonFromValue( value );
            if( button != null )
                button.setSelected( true );
                        
            firePropertyChange( "value", oldval, value );
        }
    }
    
    public int getValue()  { return _value; }
    
    
    /**
     * Re-usable code to find the appropriate button from an anchor constant.
     * Used in selecting and focusing buttons.  Will return null if an invalid
     * value is passed.  There is no need to map the other way as the
     * AnchorAction instances take care of it.
     */
    protected JToggleButton buttonFromValue( int value )
    {
        switch( value )
        {
        case Title.JUSTIFY_LEFT:   return leftBtn;
        case Title.JUSTIFY_CENTER: return centerBtn;
        case Title.JUSTIFY_RIGHT:  return rightBtn;
        }
        
        return null;
    }
    
    
    /**
     * Simple inner class to handle clicks on the buttons.  The value passed to
     * the constructor is passed to setValue().
     */
    private class JustifyAction
        implements ActionListener
    {
        private int _justification;
        
        public JustifyAction( int justification )
        {
            _justification = justification;
        }
        
        public void actionPerformed( ActionEvent e )
        {
            setValue( _justification );
            fireActionEvent( new ActionEvent( this, ActionEvent.ACTION_PERFORMED, Integer.toString(_value) ) );
        }
    }
}
