/* *****************************************************************************
 *
 * Filename: StaticImage.java
 *
 * Author:   Robert Thornburrow
 * Project:  CalendarMaker
 * Created:  09 February 2003
 *
 * Purpose:  Bean for displaying a static image.
 *
 * (c) Copyright R.H.Thornburrow, 2003
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *****************************************************************************/

package com.tsac.calmaker.beans;

import java.beans.Beans;
import java.awt.*;
import java.awt.image.*;
import java.util.ResourceBundle;
import javax.swing.JComponent;
import javax.accessibility.*;

import com.tsac.calmaker.IScaleable;
import com.tsac.calmaker.MainFrame;

public class StaticImage
    extends javax.swing.JComponent
    implements IScaleable, Accessible
{
    // Statics
    protected static final ResourceBundle resources = ResourceBundle.getBundle( "com/tsac/calmaker/beans/CalendarBeans" );
    public static final String IMAGE_PROPERTY = "image";
    
    // Properties
    protected String _filename = null;
    
    // State
    protected transient MediaTracker tracker = new MediaTracker( this );
    protected transient Image _image = null;
    protected transient ImageObserver _observer;
    protected transient int _id = 0;
    
    // Cached
    private transient Font _font = new Font( "Helvetica", Font.PLAIN, 12 );
    private transient FontMetrics _fontMetrics;
    

    // Contructor
	public StaticImage()
	{
        setFocusable( true );
        setPreferredSize( new Dimension( 100, 100 ) );
	    _fontMetrics = getFontMetrics( _font );
	    
        if( !Beans.isDesignTime() )
        {
            // For testing only!
            setImage( "images/logo.gif" );
	        
            // Need an observer to refresh the display after the image loads
	        _observer = new ImageObserver() {
	            public boolean imageUpdate( Image img, int infoflags, int x, int y, int width, int height )
	            {
	                if( (infoflags & ImageObserver.ALLBITS) != 0 )
	                {
	                    repaint();
	                    return false;
	                }
	                
	                return true;
	            }
	        };
	    }
	}


    // Property accessors
    public String getImage()  { return _filename; }
        
    public void setImage( String i )
    {
        // Suppress if no change (object comparison)
        if( _filename == i )
            return;
        
        String oldfname = _filename;
        
        // If this is not the same image as before (but with a different string object)?
        if( (i == null) || (_filename == null) || !_filename.equals(i) )
        {
            removeImage();
            _filename = i;
            _id++;
            loadImage();
            repaint();
        }
                
        // Store the new filename anyway (could be a different string)
        _filename = i;
        
        // Fire a property change
        firePropertyChange( IMAGE_PROPERTY, oldfname, _filename );
    }


    public void paint( Graphics g )
    {
        paint( g, MainFrame.scale );
    }


    // Overrides default paint method
    public void paint( Graphics g, double scale )
    {
        if( Beans.isDesignTime() )
        {
            g.setColor( Color.lightGray );
            g.fillRect( 0, 0, getWidth(), getHeight() );
            g.setColor( Color.black );
            g.drawRect( 2, 2, getWidth() - 4, getHeight() - 4 );
            g.drawString( getClass().getName(), 4, _fontMetrics.getAscent() + 4 );
            return;
        }
        
        if( _image != null )
        {
            int w = _image.getWidth( _observer );
            int h = _image.getHeight( _observer );
            
            int status = tracker.statusID( _id, true );
            
            int width  = (int) (scale * getWidth());
            int height = (int) (scale * getHeight());
            
            if( status == MediaTracker.COMPLETE )
            {
                double xscale = (double) getWidth() / (double) w;
                double yscale = (double) getHeight() / (double) h;
                        
                double actscale = Math.min( xscale, yscale );
                
                int imgwidth = (int) ((double) w * actscale * scale);
                int imgheight = (int) ((double) h * actscale * scale);
                                
                g.drawImage( _image,
                    (width - imgwidth)/2, (height - imgheight)/2,
                    imgwidth, imgheight, Color.white, _observer );
            }
            else if( status == MediaTracker.LOADING )
            {
                g.setColor( Color.black );
                g.setFont( _font );
                g.drawString( "Loading...",
                    (width - _fontMetrics.stringWidth("Loading..."))/2,
                    (height - _fontMetrics.getHeight())/2 );
            }
            else
            {
                g.setColor( Color.black );
                g.setFont( _font );
                g.drawString( "Failed!",
                    (width - _fontMetrics.stringWidth("Failed!"))/2,
                    (height - _fontMetrics.getHeight())/2 );
            }
        }
    }
        
    
    private void loadImage()
    {
        // Is the image we are supposed to load unset?
        if( _filename == null )
        {
            _image = null;
            return;
        }

        // Load the image and store it in the images array
        _image = Toolkit.getDefaultToolkit().getImage( _filename );
        tracker.addImage( _image, _id );
    }
    
    
    private void removeImage()
    {
        // Do we have an existing image to remove?
        if( _image != null )
            tracker.removeImage( _image );
        
        // Remember the fact, so that we won't remove it again
        _image = null;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    // Accessibility stuff
    
    public class AccessibleStaticImage
        extends JComponent.AccessibleJComponent
    {
        public String getAccessibleName()
        {
            // Use name set by the developer if set, otherwise use name from bean
            String name = super.getAccessibleName();            
            return (name != null) ? name : resources.getString( "StaticImage_Name" );
        }
        
        public String getAccessibleDescription()
        {
            // Use description set by the developer if set, otherwise use description from bean
            String name = super.getAccessibleDescription();            
            return (name != null) ? name : resources.getString( "StaticImage_Desc" );
        }
    }
    
    
    public AccessibleContext getAccessibleContext()
    {
        if( accessibleContext == null )
            accessibleContext = new AccessibleStaticImage();
        
        return accessibleContext;
    }
}